// ДАННЫЙ ПРИМЕР ВЫВОДИТ КООРДИНАТЫ ДЖОЙСТИКА:    //
                                                  //
#include <iostream>
#define print std::cout<<
#define println(A) std::cout<<(A)<<'\n';
#include "../iarduino_I2C_Joystick.h"             //   Подключаем библиотеку для работы с джойстиком I2C-flash.
iarduino_I2C_Joystick joy(0x09);                  //   Объявляем объект joy для работы с функциями и методами библиотеки iarduino_I2C_Joystick, указывая адрес модуля на шине I2C.
                                                  //   Если объявить объект без указания адреса (iarduino_I2C_Joystick joy;), то адрес будет найден автоматически.
void setup(){                                     //
    joy.begin();                                  //   Инициируем работу с джойстиком.
}                                                 //
                                                  //
void loop(){                                      //
//  Получаем и выводим координаты по отдельности: //
    print( "X=" );                                //
    print( joy.getPosition_X() );                 //   Выводим координату по оси X, значение от -100 до 100.
    print( ", Y=" );                              //
    print( joy.getPosition_Y() );                 //   Выводим координату по оси Y, значение от -100 до 100.
//  Получаем сразу обе координаты:                //
    int x,y;                                      //   Объявляем переменные для получения координат.
    joy.getPosition(x,y);                         //   Получаем координаты в указанные переменные.
    print( ", X:Y=" );                            //
    print( x );                                   //   Выводим координату по оси X, значение от -100 до 100.
    print( ":" );                                 //
    print( y );                                   //   Выводим координату по оси X, значение от -100 до 100.
    print( "." );                                 //
    println("Нажмите <ctrl+x> для выхода");
    delay(100);
}

int main()
{
	print("Координаты джойстика:");
	setup();
	for (;;)
		loop();
}
